//****************************************************************************************
// Program: Drehgeber auswerten mit Cortex M3 Board
//
// Author : Daniel Buergin
//
// Beschreibung: ines "Heidenhain ROD 426" Drehgebers
//
// Gesteuert ueber einen Drehgeber wird ueber einen eigenen Pin ein Laser
// gesteuert. Damit koennen in regelmaessigen Abstaenden Loecher in
// eine Platte gebohrt werden.
//
//****************************************************************************************
// Version: 0.4
//****************************************************************************************
// Aenderungen:
//
// 01-Nov-2009 bue Erste Versuche mit dem neuen Cortex-M3 Board
// 10-Nov-2009 bue LCD und USART gehen
// 12-Nov-2009 bue Drehgeber ok
//
//****************************************************************************************
//
// Copyrights: - Teilweise bei mir, teilweise auf Code-Fragmente aus dem Internet
//             - Drehgeber und Tasten Alogrithmen von Peter Danneger (danni@specs.de)
//
//***************************************************************************************
//
// Erstellt arm-elf-gcc (4.3.3) Toolchain 
//
//****************************************************************************************
// HW: ET-STM32F103 Board von www.futurlec.com
//     LCD Display mit 4x20 Zeichen mit HD44780 compatiblem Kontroller
//     ROD 426 Drehgeber von Heidenhain (www.heidenhain.com"
//         - orange/weiss = 5V
//         - blau/weiss   = 0V
//         - braun/weiss  = Ua1
//         - gruen/weiss  = Ua2
//
// Konfiguration:
// ==============
//
// - GPIOA:
//      PA0:                --> WKUP Button 
//      PA1: 
//      PA2:
//      PA3:
//      PA4:                --> SD CD/DAT3
//      PA5:                --> SD CLK
//      PA6:                --> SD DAT0
//      PA7:                --> SD CMD
//      PA8: (5V)           --> SD Card Detect
//      PA9: (5V)           --> USART1 TX
//      PA10: (5V)          --> USART1 RX
//      PA11: (5V)          --> 
//      PA12: (5V)          -->
//      PA13: (5V)          --> TMS (JTAG)
//      PA14: (5V)          --> TCK (JTAG)
//      PA15: (5V)          --> TDI (JTAG)
//
// - GPIOB:
//      PB0:                --> Button 
//      PB1:                --> Button
//      PB2: (5V)           --> Button
//      PB3: (5V) TDO       --> Button
//      PB4: (5V) TRST      --> Button 
//      PB5:                --> Button 
//      PB6: (5V)           --> Button
//      PB7: (5V)           --> Button
//      PB8: (5V)           -->
//      PB9: (5V)           --> LED
//      PB10: (5V)          --> LED
//      PB11: (5V)          --> LED
//      PB12: (5V)          --> LED
//      PB13: (5V)          --> LED
//      PB14: (5V)          --> LED
//      PB15: (5V)          --> LED
//
// - GPIOC:
//      PC0:                --> LCD D7  
//      PC1:                --> LCD D6
//      PC2:                --> LCD D5
//      PC3:                --> LCD D4
//      PC4:                --> 
//      PC5:                --> 
//      PC6: (5V)           --> SD WP
//      PC7: (5V)           -->
//      PC8: (5V)           --> Drehgeber A 
//      PC9: (5V)           --> Drehgeber B 
//      PC10: (5V)          --> LCD EN
//      PC11: (5V)          --> LCD RW
//      PC12: (5V)          --> RS
//      PC13:               --> TAMP Button
//      PC14: N/A
//      PC15: N/A
//
//****************************************************************************************



//* Includes ------------------------------------------------------------------*

#include "stm32f10x.h"
#include "platform_config.h"
#include "my_delay.h"
#include "lcd.h"
#include "usart.h"
#include "printf.h"

//* Private typedef -----------------------------------------------------------*
//* Private define ------------------------------------------------------------*
//* Private macro -------------------------------------------------------------*
//* Private variables ---------------------------------------------------------*

// Some special Characters

#define CR 13
#define LF 10
#define STX 2
#define ETX 3

#define MUE     0xe4    // Das Zeichen Mue, 1110|0100
#define GRAD    0xdf    // Das Gradzeichen, 1110|1111
#define STR     0x7c    // Laengstrich    , 0111|1100
#define BLOCK   0xff    // Schwarzer Block, 1111|1111

// #define sbi(ADDRESS,BIT) ((ADDRESS) |= (1<<(BIT)))
// #define cbi(ADDRESS,BIT) ((ADDRESS) &= ~(1<<(BIT)))
// #define outp(VAL,ADRESS) ((ADRESS) = (VAL))
// #define inp(VAL) (VAL)

// Variablen fuer die Auswertung des Drehgebers:
// ---------------------------------------------
// Es sind alles int32 Variablen, da der Drehgeber pro
// milimeter 1000 Impulse liefert. D.h. bei einer Strecke
// von 20cm ergibt das 200'000 Impulse, und das passt nicht
// mehr in einen int16 Variable.
// int32 = -2'147'483'648 bin +2'147'482'647

// Counter fuer den Drehgeber. Diese Variabel wird in der 
// Interruptservice Routine hoch- oder runtergezaehlt. Ja
// nach Drehrichtung des Drehgebers.

volatile int32_t enc_delta;

// Counter fuer die Distanz zwischen den Loechern. Hier steht die 
// Distanz zwischen dem ersten und dem zweiten Loch

volatile int32_t distanz;

// Counter fuer das letzte Loch

volatile int32_t last;

// Offset fuer den Versatz des Lochs bei hoher Geschwindigkeit

volatile int32_t offset;

// Diese Variable wird zwischen jedem Loch hochgezaehlt. Diese Variable
// wird ebenfalls von der Interruptservice-routine hoch- und runter
// getzaehlt. Aber bei jedem "Schuss" auf 0 gesetzt.

volatile int32_t delta;

// Damit wird der aeussere Rand des "Schussfeldes" definiert. Ausserhalb
// dieser Grenzen ist die "fire" Variabel auf Null und der Laser
// erhaelt keine Impulse, auch wenn die Distanz einen Schuss ergaebe

volatile int32_t upper_limit;
volatile int32_t lower_limit;

// Statusvarible fuer die Buttonroutine

uint8_t state;

// Variable die festlegt, ober der Laser getriggerd wird oder nicht.

uint8_t fire;

//=====================================================================================
// Displaybuffer fuer die Anzeige
//=====================================================================================

char disp_buffer[21];
char buffer[64];

//=====================================================================================
// Variabeln fuer die Buttons
//=====================================================================================

#define REPEAT_MASK (1<<BUTTON1^1<<BUTTON2^1<<BUTTON3^1<<BUTTON4^1<<BUTTON5)
// #define REPEAT_MASK (1<<B1)
#define REPEAT_START    50              // after 500ms
#define REPEAT_NEXT     20              // every 200ms

// bit = 1: key pressed
uint16_t key_state;
// key long press and repeat
uint16_t key_rpt;
// key press detect
uint16_t key_press;

TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
NVIC_InitTypeDef         NVIC_InitStructure;
GPIO_InitTypeDef         GPIO_InitStructure;
ErrorStatus HSEStartUpStatus;

//* Private function prototypes -----------------------------------------------*

void SystemInitialization(void);

/* Private functions ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private typedef -----------------------------------------------------------*/

/*******************************************************************************
* Function Name  : main
* Description    : Main program.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/

int main(void)
{
  
  uint8_t enc_a = 0;
  uint8_t enc_b = 0;

#ifdef DEBUG
  debug();
#endif

  SystemInitialization();

  usart1_init(9600);

  // Die eigene printf/sprintf Umgebung initialisieren
  // Dazu wird der Pointer auf die entsprechende 
  // Zeichenausgaberoutine der Funktion (printf.c) 
  // uebergeben

  init_printf(0x0, usart1_Xputc);

  // Timer 1 wird fuer die Encoder aufgesetzt.
  // Frequenz: 400khz
  // Prescaler = 36
  // Period    = 4
  // Sollte ergeben: 72Mhz / 36 = 2Mhz / 5 = 400khz
  TIM_TimeBaseStructure.TIM_Prescaler = 36;
  TIM_TimeBaseStructure.TIM_Period = 4;

  // TIM1 counter setup fuer 1s
  // TIM_TimeBaseStructure.TIM_Prescaler = 7200;
  // TIM_TimeBaseStructure.TIM_Period = 9999;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;

  TIM_TimeBaseInit(TIM1, &TIM_TimeBaseStructure);

  TIM_ITConfig(TIM1, TIM_IT_Update, ENABLE);

  /* TIM1 counter enable */
  TIM_Cmd(TIM1, ENABLE);

  /* Setup SysTick Timer for 1 msec interrupts  */
  if (SysTick_Config(SystemFrequency / 1000))
  {
    /* Capture error */
    while (1);
  }

  // Print Message to LCD 20x4 //

  lcd_init();
 
  lcd_gotoxy(0,0);
  lcd_print("ST-ARM Cortex-M3    "); // Display LCD Line-1
  lcd_gotoxy(0,1);
  lcd_print("Codesourcery GCC    "); // Display LCD Line-2
  lcd_gotoxy(0,2);
  lcd_print("                    "); // Display LCD Line-3    
  lcd_gotoxy(0,3);
  lcd_print("BY...... D.Buergin  "); // Display LCD Line-4
  Delay(2500);                       // Display Delay

  lcd_gotoxy(0,3);
  lcd_print("                    "); // Display LCD Line-4

  sprintf(buffer,"%s","Starting up....\n\r");
  usart1_puts(buffer);

  // Hauptschleife

  while(1)
  {

	// Der Encoder wird in der TIM1 Routine abgefragt.
	// Hier wird nur der Zaehler angezeigt.

	sprintf(disp_buffer,"Enc: %09d",enc_delta);
    lcd_gotoxy(0,3);
	lcd_print(disp_buffer);
  
    // sprintf(buffer,"%s","I'am alive\n\r");
	// usart1_puts(buffer);

    Delay(50);
  }  
}

//---------------------------------------------------------------------------
// Meine eigene System Init Routine
// Eigentlich nur eine Zusammenfassung von RCC, NVIC und GPIO Stuff
//---------------------------------------------------------------------------

void SystemInitialization(void)
{

  //---------------------------------------------------------------------------
  // Generic RCC Stuff
  //---------------------------------------------------------------------------

  /* RCC system reset(for debug purpose) */
  RCC_DeInit();

  /* Enable HSE */
  RCC_HSEConfig(RCC_HSE_ON);

  /* Wait till HSE is ready */
  HSEStartUpStatus = RCC_WaitForHSEStartUp();

  if(HSEStartUpStatus == SUCCESS)
  {
    /* Enable Prefetch Buffer */
    FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

    /* Flash 2 wait state */
    FLASH_SetLatency(FLASH_Latency_2);
 	
    /* HCLK = SYSCLK */
    RCC_HCLKConfig(RCC_SYSCLK_Div1); 
  
    /* PCLK2 = HCLK */
    RCC_PCLK2Config(RCC_HCLK_Div1); 

    /* PCLK1 = HCLK/2 */
    RCC_PCLK1Config(RCC_HCLK_Div2);

    /* PLLCLK = 8MHz * 9 = 72 MHz */
    RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);

    /* Enable PLL */ 
    RCC_PLLCmd(ENABLE);

    /* Wait till PLL is ready */
    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET) {
    }

    /* Select PLL as system clock source */
    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

    /* Wait till PLL is used as system clock source */
    while(RCC_GetSYSCLKSource() != 0x08) {
	}

  }

  /* TIM1 clock enable */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM1,ENABLE);

  //---------------------------------------------------------------------------
  // NVIC Stuff
  //---------------------------------------------------------------------------

#ifdef  VECT_TAB_RAM
  /* Set the Vector Table base location at 0x20000000 */
  NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0);
#else  /* VECT_TAB_FLASH  */
  /* Set the Vector Table base location at 0x08000000 */
  NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);
#endif

  /* Enable the EXTI0 Interrupt */
  NVIC_InitStructure.NVIC_IRQChannel = TIM1_UP_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

  /* Enable the USART1 Interrupt */
  NVIC_InitStructure.NVIC_IRQChannel = USART1_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

  //---------------------------------------------------------------------------
  // Configure all unused GPIO port pins in Analog Input mode (floating input
  // trigger OFF), this will reduce the power consumption and increase the device
  // immunity against EMI/EMC
  //---------------------------------------------------------------------------

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB |
                         RCC_APB2Periph_GPIOC | RCC_APB2Periph_GPIOD |
                         RCC_APB2Periph_GPIOE, ENABLE);

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_All;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
  GPIO_Init(GPIOB, &GPIO_InitStructure);
  GPIO_Init(GPIOC, &GPIO_InitStructure);
  GPIO_Init(GPIOD, &GPIO_InitStructure);
  GPIO_Init(GPIOE, &GPIO_InitStructure);

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB |
                         RCC_APB2Periph_GPIOC | RCC_APB2Periph_GPIOD |
                         RCC_APB2Periph_GPIOE, DISABLE);

  //---------------------------------------------------------------------------
  // LED's on GPIOB Pin 8,9,10,11,12,13,14,15
  //---------------------------------------------------------------------------

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIO_LED, ENABLE);

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(GPIO_LED, &GPIO_InitStructure);

  //---------------------------------------------------------------------------
  // GPIOC Port 8 und 9 aktivieren, wird vom Drehgeber verwendet
  //---------------------------------------------------------------------------

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIO_ENC, ENABLE);
  GPIO_InitStructure.GPIO_Pin = ENC1_UA1 | ENC1_UA2;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(GPIO_ENC, &GPIO_InitStructure);

  // Disable JTAG-Port
  // GPIO_PinRemapConfig(GPIO_Remap_SWJ_JTAGDisable, ENABLE);
}

#ifdef  DEBUG

/*******************************************************************************
* Function Name  : assert_failed
* Description    : Reports the name of the source file and the source line number
*                  where the assert_param error has occurred.
* Input          : - file: pointer to the source file name
*                  - line: assert_param error line source number
* Output         : None
* Return         : None
*******************************************************************************/

void assert_failed(u8* file, u32 line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif
